<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       example.com
 * @since      1.0.0
 *
 * @package    Learndash_Flashcards
 * @subpackage Learndash_Flashcards/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Learndash_Flashcards
 * @subpackage Learndash_Flashcards/includes
 * @author     Srikanth <srikanthkoneru631@gmail.com>
 */
class Learndash_Flashcards {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Learndash_Flashcards_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		if ( defined( 'LEARNDASH_FLASHCARDS_VERSION' ) ) {
			$this->version = LEARNDASH_FLASHCARDS_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = 'learndash-flashcards';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Learndash_Flashcards_Loader. Orchestrates the hooks of the plugin.
	 * - Learndash_Flashcards_i18n. Defines internationalization functionality.
	 * - Learndash_Flashcards_Admin. Defines all hooks for the admin area.
	 * - Learndash_Flashcards_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-learndash-flashcards-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-learndash-flashcards-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-learndash-flashcards-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-learndash-flashcards-public.php';

		/**
		 * The class responsible for adding shortcodes
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-shortcode.php';			

		/**
		 * The class responsible for adding blocks
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-blocks.php';			

		$this->loader = new Learndash_Flashcards_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Learndash_Flashcards_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Learndash_Flashcards_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Learndash_Flashcards_Admin( $this->get_plugin_name(), $this->get_version() );
		$plugin_blocks = new Learndash_Flashcards_Blocks( $this->get_plugin_name(), $this->get_version() );
		$plugin_shortcodes = new Learndash_Flashcards_Shortcodes();

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );
		$this->loader->add_action( 'init', $plugin_admin, 'create_flashcard_post_type' );
		$this->loader->add_filter('learndash_submenu', $plugin_admin, 'add_coupon_admin_menu', 1);
		$this->loader->add_action( 'init', $plugin_admin, 'create_flashcards_taxonomy', 0 );
		$this->loader->add_action( 'add_meta_boxes', $plugin_admin, 'learndash_flashcards_add_meta_box' );
		$this->loader->add_action( 'save_post', $plugin_admin, 'learndash_flashcards_save_meta', 10, 3 );
		
		/* Setup Blocks */
		$this->loader->add_action( 'enqueue_block_editor_assets', $plugin_blocks, 'admin_block_assets' );
		$this->loader->add_action( 'enqueue_block_assets', $plugin_blocks, 'block_assets' );
		$this->loader->add_action( 'init', $plugin_blocks, 'register_learndash_flashcards_blocks' );
		$this->loader->add_filter( 'block_categories', $plugin_blocks, 'skeleton_block_categories', 10, 2 );	
		
		/* Shortcodes */
		$this->loader->add_shortcode( 'learndash_flashcard', $plugin_shortcodes, 'learndash_flashcard_func' );
		$this->loader->add_shortcode( 'learndash_flashcard_set', $plugin_shortcodes, 'learndash_flashcard_set_func' );

	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new Learndash_Flashcards_Public( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );

	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Learndash_Flashcards_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

}
